@@ -47,6 +47,9 @@ public:
     STD_WEAK_PTR<OHTime> ohtm;
     STD_WEAK_PTR<OHVolume> ohvl;
     STD_WEAK_PTR<OHReceiver> ohrc;
+    STD_WEAK_PTR<OHRadio> ohrd;
+    STD_WEAK_PTR<OHInfo> ohif;
+    STD_WEAK_PTR<OHSender> ohsn;
 };
 
 // We don't include a version in comparisons, as we are satisfied with
@@ -204,6 +207,60 @@ OHRCH MediaRenderer::ohrc()
     return ohrcl;
 }
 
+OHRDH MediaRenderer::ohrd() 
+{
+    OHRDH handle = m->ohrd.lock();
+    if (handle)
+        return handle;
+    for (vector<UPnPServiceDesc>::const_iterator it = desc()->services.begin();
+         it != desc()->services.end();it++) {
+        if (OHRadio::isOHRdService(it->serviceType)) {
+            handle = OHRDH(new OHRadio(*desc(), *it));
+            break;
+        }
+    }
+    if (!handle)
+        LOGDEB("MediaRenderer: OHRadio service not found" << endl);
+    m->ohrd = handle;
+    return handle;
+}
+
+OHIFH MediaRenderer::ohif() 
+{
+    OHIFH handle = m->ohif.lock();
+    if (handle)
+        return handle;
+    for (vector<UPnPServiceDesc>::const_iterator it = desc()->services.begin();
+         it != desc()->services.end();it++) {
+        if (OHInfo::isOHInfoService(it->serviceType)) {
+            handle = OHIFH(new OHInfo(*desc(), *it));
+            break;
+        }
+    }
+    if (!handle)
+        LOGDEB("MediaRenderer: OHInfo service not found" << endl);
+    m->ohif = handle;
+    return handle;
+}
+
+OHSNH MediaRenderer::ohsn() 
+{
+    OHSNH handle = m->ohsn.lock();
+    if (handle)
+        return handle;
+    for (vector<UPnPServiceDesc>::const_iterator it = desc()->services.begin();
+         it != desc()->services.end();it++) {
+        if (OHSender::isOHSenderService(it->serviceType)) {
+            handle = OHSNH(new OHSender(*desc(), *it));
+            break;
+        }
+    }
+    if (!handle)
+        LOGDEB("MediaRenderer: OHSender service not found" << endl);
+    m->ohsn = handle;
+    return handle;
+}
+
 OHTMH MediaRenderer::ohtm() 
 {
     OHTMH ohtml = m->ohtm.lock();
