@@ -15,23 +15,26 @@
  *       59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "libupnpp/control/ohproduct.hxx"
+#include "libupnpp/config.h"
 
-#include <expat_external.h>             // for XML_Char
 #include <string.h>                     // for strcmp
 #include <upnp/upnp.h>                  // for UPNP_E_BAD_RESPONSE, etc
 #include <ostream>                      // for endl
 #include <string>                       // for string
 #include <vector>                       // for vector
 
+#include "libupnpp/control/ohproduct.hxx"
 #include "libupnpp/expatmm.hxx"         // for inputRefXMLParser
 #include "libupnpp/log.hxx"             // for LOGERR
 #include "libupnpp/soaphelp.hxx"        // for SoapIncoming, etc
 #include "libupnpp/upnpp_p.hxx"         // for stringToBool, trimstring
 
 using namespace std;
+using namespace STD_PLACEHOLDERS;
 using namespace UPnPP;
 
+namespace UPnPClient {
+
 // SourceList XML format
 // <SourceList>
 //    <Source>
@@ -49,8 +52,6 @@ using namespace UPnPP;
 // - Analog - Specifies an analog external input
 // - Digital - Specifies a digital external input
 // - Hdmi - Specifies a HDMI external input
-
-namespace UPnPClient {
 class OHSourceParser : public inputRefXMLParser {
 public:
     OHSourceParser(const string& input, vector<OHProduct::Source>& sources)
@@ -105,6 +106,37 @@ bool OHProduct::isOHPrService(const string& st)
     return !SType.compare(0, sz, st, 0, sz);
 }
 
+
+void OHProduct::evtCallback(
+    const STD_UNORDERED_MAP<std::string, std::string>& props)
+{
+    LOGDEB1("OHProduct::evtCallback: getReporter(): " << getReporter() << endl);
+    for (STD_UNORDERED_MAP<std::string, std::string>::const_iterator it = 
+             props.begin(); it != props.end(); it++) {
+        if (!getReporter()) {
+            LOGDEB1("OHProduct::evtCallback: " << it->first << " -> " 
+                    << it->second << endl);
+            continue;
+        }
+        if (!it->first.compare("SourceIndex")) {
+            getReporter()->changed(it->first.c_str(), atoi(it->second.c_str()));
+        } else if (!it->first.compare("Standby")) {
+            bool val = false;
+            stringToBool(it->second, &val);
+            getReporter()->changed(it->first.c_str(), val ? 1 : 0);
+        } else {
+            LOGDEB1("OHProduct event: unknown variable: name [" <<
+                    it->first << "] value [" << it->second << endl);
+            getReporter()->changed(it->first.c_str(), it->second.c_str());
+        }
+    }
+}
+
+void OHProduct::registerCallback()
+{
+    Service::registerCallback(bind(&OHProduct::evtCallback, this, _1));
+}
+    
 int OHProduct::getSources(vector<Source>& sources)
 {
     SoapOutgoing args(getServiceType(), "SourceXml");
@@ -142,5 +174,15 @@ int OHProduct::setSourceIndex(int index)
     return runSimpleAction("SetSourceIndex", "Value", index);
 }
 
+int OHProduct::standby(bool *value)
+{
+    return runSimpleGet("Standby", "Value", value);
+}
+
+int OHProduct::setStanby(bool value)
+{
+    return runSimpleAction("SetStandby", "Value", value);
+}
+    
 
 } // End namespace UPnPClient
