@@ -16,13 +16,12 @@
  */
 #include "libupnpp/config.h"
 
-#include "device.hxx"
 
 #include <errno.h>                      // for ETIMEDOUT, errno
-#include <sys/time.h>                   // for CLOCK_REALTIME
 #include <time.h>                       // for timespec, clock_gettime
 
 #include <iostream>                     // for endl, operator<<, etc
+#include <sstream>
 #include <utility>                      // for pair
 
 #include "libupnpp/log.hxx"             // for LOGERR, LOGFAT, LOGDEB, etc
@@ -31,6 +30,7 @@
 #include "libupnpp/upnpputils.hxx"      // for timespec_addnanos
 #include "libupnpp/upnpp_p.hxx"
 #include "vdir.hxx"                     // for VirtualDir
+#include "device.hxx"
 
 using namespace std;
 using namespace UPnPP;
@@ -406,8 +406,14 @@ void UpnpDevice::Internal::notifyEvent(const string& serviceId,
                                          const vector<string>& names, 
                                          const vector<string>& values)
 {
+//    LOGDEB1("UpnpDevice::notifyEvent " << serviceId << " " <<
+//           (names.empty() ? "Empty names??" : names[0]) << endl);
+    stringstream ss;
+    for (unsigned int i = 0; i < names.size() && i < values.size(); i++) {
+        ss << names[i] << "=" << values[i] << " ";
+    }
     LOGDEB1("UpnpDevice::notifyEvent " << serviceId << " " <<
-           (names.empty()?"Empty names??":names[0]) << endl);
+            (names.empty() ? "Empty names??" : ss.str()) << endl);
     if (names.empty())
         return;
     vector<const char *> cnames, cvalues;
@@ -418,7 +424,8 @@ void UpnpDevice::Internal::notifyEvent(const string& serviceId,
                          serviceId.c_str(), &cnames[0], &cvalues[0],
                          int(cnames.size()));
     if (ret != UPNP_E_SUCCESS) {
-        LOGERR(lib->errAsString("UpnpDevice::notifyEvent", ret) << endl);
+        LOGERR(lib->errAsString("UpnpDevice::notifyEvent: id", ret) << 
+               " for " << serviceId << endl);
     }
 }
 
@@ -428,24 +435,12 @@ int timespec_diffms(const struct timespec& old, const struct timespec& recent)
         (recent.tv_nsec - old.tv_nsec) / (1000 * 1000);
 }
 
-#ifndef CLOCK_REALTIME
-// Mac OS X for one does not have clock_gettime. Gettimeofday is more than
-// enough for our needs.
-#define CLOCK_REALTIME 0
-int clock_gettime(int /*clk_id*/, struct timespec* t) {
-    struct timeval now;
-    int rv = gettimeofday(&now, NULL);
-    if (rv) return rv;
-    t->tv_sec  = now.tv_sec;
-    t->tv_nsec = now.tv_usec * 1000;
-    return 0;
-}
-#endif // ! CLOCK_REALTIME
-
 // Loop on services, and poll each for changed data. Generate event
-// only if changed data exists. Every now and then, we generate an
-// artificial event with all the current state. This is normally run
-// by the main thread.
+// only if changed data exists.
+// We used to generate an artificial event with all the current state
+// every 10 S or so. This does not seem to be necessary, and we do not
+// do it any more.
+// This is normally run by the main thread.
 void UpnpDevice::eventloop()
 {
     if (!m->start()) {
@@ -467,8 +462,7 @@ void UpnpDevice::eventloop()
     bool didearly = false;
 
     for (;;) {
-        clock_gettime(CLOCK_REALTIME, &wkuptime);
-
+        timespec_now(&wkuptime);
         timespec_addnanos(&wkuptime, loopwait_ms * 1000 * 1000);
 
         //LOGDEB("eventloop: now " << time(0) << " wkup at "<< 
@@ -523,14 +517,23 @@ void UpnpDevice::eventloop()
         for (vector<string>::iterator it = m->serviceids.begin(); 
              it != m->serviceids.end(); it++) {
             vector<string> names, values;
+            UpnpService* serv = m->servicemap[*it];
             {
+
+                // We don't generate periodic full event data any more
+                // by default. Code kept around for easier switch back
+                // on in case it's needed again
+#ifndef LIBUPNPP_PERIODIC_FULL_DEVICE_EVENTS
+                all = false;
+#endif
+                
                 PTMutexLocker lock(m->devlock);
-                UpnpService* serv = m->servicemap[*it];
                 if (!serv->getEventData(all, names, values) || names.empty()) {
                     continue;
                 }
             }
-            m->notifyEvent(*it, names, values);
+            if (!serv->noevents())
+                m->notifyEvent(*it, names, values);
         }
     }
 }
@@ -555,6 +558,12 @@ void UpnpDevice::shouldExit()
 }
 
 
+struct UpnpService::Internal {
+    Internal(bool noevs) 
+    : noevents(noevs) {
+    }
+    bool noevents;
+};
 
 UpnpService::UpnpService(const std::string& stp,
                          const std::string& sid, UpnpDevice *dev)
@@ -563,8 +572,24 @@ UpnpService::UpnpService(const std::string& stp,
     dev->addService(this, sid);
 }
 
+UpnpService::UpnpService(const std::string& stp,
+                         const std::string& sid, UpnpDevice *dev, bool noevs)
+    : m_serviceType(stp), m_serviceId(sid), m(new Internal(noevs))
+{
+    dev->addService(this, sid);
+}
+
 UpnpService::~UpnpService() 
 {
+    if (m) {
+        delete m;
+        m = 0;
+    }
+}
+
+bool UpnpService::noevents()
+{
+    return m && m->noevents;
 }
 
 bool UpnpService::getEventData(bool all, std::vector<std::string>& names, 
